@@ -1,7 +1,7 @@
 /* options.h
 
 Copyright (C) 1999-2003 Tom Gilbert.
-Copyright (C) 2010-2011 Daniel Friesel.
+Copyright (C) 2010-2018 Daniel Friesel.
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to
@@ -111,6 +111,9 @@ struct __fehoptions {
 	int geom_y;
 	unsigned int geom_w;
 	unsigned int geom_h;
+	int offset_flags;
+	int offset_x;
+	int offset_y;
 	int default_zoom;
 	int zoom_mode;
 	unsigned char adjust_reload;
